#!/usr/bin/perl -w

# getUnique.pl v0.2

# Program to replace multiple instances having the same instance id in
# the input SENSEVAL-2 formatted file with a single instance having
# multiple answers.

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Satanjeev Banerjee, University of Minnesota, Duluth
# bane0025@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

# we have to use commandline options, so use the necessary package!
use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
GetOptions("help", "version");

# if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

$file = shift; 
open (SRC, $file) || die "Couldnt open $file";

# first get hold of all the instances that have multiple answers.
while (<SRC>)
{
    if (!(/<answer instance=\"([^\"]*)\" senseid=\"([^\"]*)\"/)) { next; }

    my $instanceId = $1;
    my $senseId = $2;

    if (defined $answers{$instanceId})
    {
	$repeatedAnswers{$instanceId} = 1;
	$answers{$instanceId} .= " $senseId";
    }
    else { $answers{$instanceId} = "$senseId"; }
}
close SRC;

# now replace all the values of the hash %repeatedAnswers with the
# actual repeated answers.
foreach (keys %repeatedAnswers)
{
    $repeatedAnswers{$_} = $answers{$_};
}

# now reopen the file and output each repeated instance only once!
open (SRC, $file) || die "Couldnt open $file";

my $ignoreInstance = 0;
my $instanceId;
while (<SRC>)
{
    if (!$ignoreInstance)
    {
	if (/<instance id=\"([^\"]*)\"/) 
	{ 
	    $instanceId = $1;

	    if (defined $repeatedAnswers{$instanceId} && $repeatedAnswers{$instanceId} eq "done")
	    {
		$ignoreInstance = 1;
		next; 
	    }
	}

	if (/<answer/)
	{
	    if (defined $repeatedAnswers{$instanceId})
	    {
		if ($repeatedAnswers{$instanceId} ne "done")
		{
		    my @senseIds = split / /, $repeatedAnswers{$instanceId};
		    foreach $id (@senseIds)
		    {
			print "<answer instance=\"$instanceId\" senseid=\"$id\"/>\n";
		    }
		    $repeatedAnswers{$instanceId} = "done";
		}
		next; 
	    }
	}

	print;
    }
    else
    {
	if (/<\/instance/) { $ignoreInstance = 0; }
    }
}
close SRC;

# thats that!

# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print "Usage: getUnique.pl [OPTIONS] FILE\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "Usage: getUnique.pl [OPTIONS] FILE\n\n";

    print "Removes multiple occurrences of the same instance in FILE with a single instance,\n";
    print "containing multiple answer-tags. Output is sent to stdout.\n\n";

    print "OPTIONS:\n\n";

    print "  --help                   Prints this help message.\n\n";

    print "  --version                Prints the version number.\n\n";
}

# function to output the version number
sub showVersion
{
    print "getUnique.pl  -  Version 0.2\n";
    print "Copyright (C) 2000-2002, Ted Pedersen & Satanjeev Banerjee\n";
}

# function to output "ask for help" message when the user's goofed up!
sub askHelp
{
    print STDERR "Type getUnique.pl --help for help.\n";
}
